const hapiAuthCookie = require('hapi-auth-cookie');
const root = require('requirefrom')('');
const basicAuth = root('server/lib/basic_auth');
const validateConfig = root('server/lib/validate_config');
const getCalculateExpires = root('server/lib/get_calculate_expires');
const createScheme = root('server/lib/login_scheme').default;

module.exports = (kibana) => new kibana.Plugin({
  name: 'shield',
  require: ['elasticsearch'],

  config(Joi) {
    return Joi.object({
      enabled: Joi.boolean().default(true),
      cookieName: Joi.string().default('sid'),
      encryptionKey: Joi.string(),
      sessionTimeout: Joi.number().default(30 * 60 * 1000),
      // Only use this if SSL is still configured, but it's configured outside of the Kibana server
      // (e.g. SSL is configured on a load balancer)
      skipSslCheck: Joi.boolean().default(false)
    }).default()
  },

  uiExports: {
    chromeNavControls: ['plugins/shield/views/logout_button'],
    apps: [{
      id: 'login',
      title: 'Login',
      main: 'plugins/shield/views/login',
      hidden: true,
      autoload: kibana.autoload.styles
    }, {
      id: 'logout',
      title: 'Logout',
      main: 'plugins/shield/views/logout',
      hidden: true,
      autoload: kibana.autoload.styles
    }]
  },

  init(server, options) {
    const config = server.config();
    validateConfig(config);

    server.register(hapiAuthCookie, (error) => {
      if (error != null) throw error;

      server.auth.scheme('login', createScheme({
        redirectUrl: (path) => loginUrl(config.get('server.basePath'), path),
        strategy: 'shield'
      }));

      server.auth.strategy('session', 'login', 'required');

      server.auth.strategy('shield', 'cookie', false, {
        cookie: config.get('shield.cookieName'),
        password: config.get('shield.encryptionKey'),
        path: config.get('server.basePath') + '/',
        clearInvalid: true,
        validateFunc: root('server/lib/validate')(server)
      });
    });

    basicAuth.register(server, config.get('shield.cookieName'));

    root('server/routes/api/v1/authenticate')(server);
    root('server/routes/views/login')(server, this);
    root('server/routes/views/logout')(server, this);
  }
});

function loginUrl(baseUrl, requestedPath) {
  const next = encodeURIComponent(requestedPath);
  return `${baseUrl}/login?next=${next}`;
}
